use crate::error::{ProcessError, Result};
use crate::{FILES_TO_CONVERT, FILES_TO_COPY, PATHS_TO_IGNORE};
use std::path::{Path, PathBuf};

#[derive(Debug, PartialEq, Eq, Clone)]
pub enum FileType {
    Convert { source: PathBuf, target: PathBuf },
    Copy { source: PathBuf, target: PathBuf },
    Ignore,
    Other,
}

impl FileType {
    /// Determines the type of a file and its processing requirements
    pub fn from_path(path: &Path, source_base: &Path, target_base: &Path) -> Result<Self> {
        // First check if it should be ignored
        if Self::is_ignored(path) {
            return Ok(FileType::Ignore);
        }

        let relative = path
            .strip_prefix(source_base)
            .map_err(ProcessError::StripPrefix)?;

        let source = path.to_path_buf();
        let target = target_base.join(relative);

        // Then check file extension
        match path
            .extension()
            .and_then(|e| e.to_str())
            .map(|ext| ext.to_lowercase())
        {
            Some(ext) if FILES_TO_CONVERT.contains(&ext.as_str()) => Ok(FileType::Convert {
                source,
                target: target.with_extension("pdf"),
            }),
            Some(ext) if FILES_TO_COPY.contains(&ext.as_str()) => {
                Ok(FileType::Copy { source, target })
            }
            _ => Ok(FileType::Other),
        }
    }

    /// Check if a path should be ignored
    fn is_ignored(path: &Path) -> bool {
        path.file_name()
            .and_then(|n| n.to_str())
            .map(|name| PATHS_TO_IGNORE.iter().any(|&ignore| name.contains(ignore)))
            .unwrap_or(false)
    }

    /// Get the source path if this is a processable file type
    pub fn source(&self) -> Option<&Path> {
        match self {
            FileType::Convert { source, .. } | FileType::Copy { source, .. } => Some(source),
            _ => None,
        }
    }

    /// Returns true if this file type should be processed
    pub fn should_process(&self) -> bool {
        matches!(self, FileType::Convert { .. } | FileType::Copy { .. })
    }

    /// Returns true if this file type should be ignored
    pub fn should_ignore(&self) -> bool {
        matches!(self, FileType::Ignore)
    }
}
